package l12_generics;

import utilities.RandomGenerators;
import java.util.LinkedList;
import java.util.List;

public class BoundedWildcards {	

	//	To write the method that works on lists of Integer and the supertypes of
	//	Integer, such as Integer, Number, and Object, you would specify
	//	List<? super Integer>. The term List<Integer> is more restrictive than
	//	List<? super Integer> because the former matches a list of type Integer only,
	//	whereas the latter matches a list of any type that is a supertype of
	//	Integer. List<Number> and List<Object> can also hold integers, so we
	//	can use lower bound wildcard to achieve this.

	// This list can hold Integer elements and elements of any base class of
	// Integer (Number, Object)
	private static void addIntegers(List<? super Integer> list) {
		for (int i = 1; i <= 8; i++) {
			list.add(RandomGenerators.getInt(100));
		}
	}

	//	static double calcAvg(List<Number> list) {
	//	error: List<Integer> cannot be converted to List<Number> (List<Integer>
	//	is NOT a subclass of List<Number> even if <Integer> is a subclass of
	//	<Number>!!!

	// This list can hold elements of class Number or any subclass of Number
	private static double calculateAverage(List<? extends Number> list) {
		double average = 0.;
		for (Number number : list) average += number.doubleValue();
		return average / list.size();
	}

//	you can print any list ...
	private static void printList(List<?> l, String type) {
		System.out.print(type + ": ");
		for(Object e : l) {
			System.out.print(e + " ");
		}
		System.out.println();
	}

	public static void main(String[] args) {

		List<Number> listOfNumbers = new LinkedList<>();
		addIntegers(listOfNumbers);
		printList(listOfNumbers, "List<Number>");
		System.out.println("The average is: " + calculateAverage(listOfNumbers));
		System.out.println();

		List<Integer> listOfIntegers = new LinkedList<>();
		addIntegers(listOfIntegers);
		printList(listOfIntegers, "List<Integer>");
		System.out.println("The average is: " + calculateAverage(listOfIntegers));
		System.out.println();

		List<Object> listOfObjects = new LinkedList<>();
		addIntegers(listOfObjects);
		printList(listOfObjects, "List<Object>");

//		error! Object does not extend Number
//		System.out.println("The average is: " + calcAvg(listOfObjects));
	}
}

