package l12_generics;

public class BoxDemo {

//	ONLY place Integer objects into this box!
//	We know that the cast from Object to Integer is correct because we've honored
//	the "contract" specified in the comment. But remember, the compiler knows
//	nothing about this; it just trusts that our cast is correct. Furthermore, it
//	will do nothing to prevent a careless programmer from passing in an object of
//	the wrong type

//	There is no contract that could guarantee that the return type of the object
//	field of the Box class is an Integer. The defined class could hold any
//	object. When looking at types, it can only guarantee that it is an Object,
//	thus requires an explicit cast to ensure that the type is safe. This cast
//	can be annoying, we know that the data type in this object is an Integer.
//	The cast is also cluttering our code. It can cause type-related runtime
//	errors if a programmer makes a mistake with the explicit casting. It would
//	be much easier if programmers could express their intention of using
//	specific types and the compiler can ensure the correctness of such type.
//	This is the core idea behind generics.

	record Box(Object object) { }
	
	public static void main(String[] args) {
		Box integerBox = new Box(10);
		Integer someInteger = (Integer)integerBox.object(); // casting necessary!!!
		System.out.println(someInteger);
		System.out.println();

//	Imagine this is one part of a large application
//	created by one programmer.
		Box box = new Box("10"); // note how the type is now String
//	and this is another, perhaps written by a different programmer
		Integer someInt = (Integer)box.object();
		System.out.println(someInt);
	}
}
