package l12_generics;

/*
	Generics are a facility of generic programming that were added to the Java
	programming language in 2004 within version J2SE 5.0

	You can think of a generic type invocation as being similar to an ordinary
	method invocation, but instead of passing an argument to a method, you're
	passing a type argument (Integer in this case) to the Box class itself.

	Like any other variable declaration, this code does not actually create a new
	Box object. It simply declares that integerBox will hold a reference to a "Box
	of Integer", which is how Box<Integer> is read.

	An invocation of a generic type is generally known as a parameterized type.
	Generics were introduced to the Java language to provide tighter type checks
	at compile time and to support generic programming.

	Replace all type parameters in generic types with their bounds or Object if
	the type parameters are unbounded. The produced bytecode, therefore,
	contains only ordinary classes, interfaces, and methods.

	By adding the diamond operator <> containing the type, we narrow the
	specialization of this list only to Integer type i.e. we specify the type
	that will be held inside the list. The compiler can enforce the type at
	compile time. In small programs, this might seem like a trivial addition,
	however, in larger programs, this can add significant robustness and makes
	the program easier to read.

	Type Parameter Naming Conventions
	By convention, type parameter names are single, uppercase letters. This
	stands in sharp contrast to the variable naming conventions that you already
	know about, and with good reason: Without this convention, it would be
	difficult to tell the difference between a type variable and an ordinary
	class or interface name.

	The most commonly used type parameter names are:
	E - Element (used extensively by the Java Collections Framework)
	K - Key
	N - Number
	T - Type
	V - Value
	S,U,V etc. - 2nd, 3rd, 4th types
*/

public class GenericBoxDemo {

	record Box<T>(T t) { }

	public static void main(String[] args) {
		Box<Integer> integerBox = new Box<>(5);
		Integer someInteger = integerBox.t();	//	no cast!
		System.out.println(someInteger);

		Box<String> StringBox = new Box<>("String");
		String someString = StringBox.t();
		System.out.println(someString);
	}
}

