package l12_generics;

public class GenericInterfaceExample {

	interface GenericInterface<T> {
		T performOperation(T value);
	}

	public static void main(String[] args) {

		// Create an instance of the generic interface
		GenericInterface<Object> genericInterface = new GenericInterface<>() {
			@Override
			public Object performOperation(Object value) {
				// Java now allows you to switch in the manner of the OP.
				// They call it Pattern Matching for switch.
				// It was released as a preview feature of Java 17
				return switch (value) {
					case Integer i -> i * 2;
					case Double v -> v / 2;
					case String s -> s.toUpperCase();
					case null, default -> null;
				};
			}
		};

		// Perform the operation on the input values
		Object result1 = genericInterface.performOperation(5);
		Object result2 = genericInterface.performOperation(9.);
		Object result3 = genericInterface.performOperation("stringValue");
		Object result4 = genericInterface.performOperation(null);
		Object result5 = genericInterface.performOperation(9.f);

		// Display the results
		System.out.println("Result 1: " + result1);
		System.out.println("Result 2: " + result2);
		System.out.println("Result 3: " + result3);
		System.out.println("Result 4: " + result4);
		System.out.println("Result 5: " + result5);
	}
}
